/*
 *  LED_Remote_Color_Picker_Sender
 *  Reads 10K pts for red, green, blue, sets a local LED to the color
 *  diplays the RGB values on an OLED display, and sends the command
 *  to the receiver on the mesh
 *  
 *  On the shoulders of giants - uses PainlessMesh and FastLED, 
 *  and the Adafruit OLED and ADC libraries
 *  
 *  2021 Carl F Sutter
 *  
 */

// OLED Setup
#include <SPI.h>
#include <Wire.h>
#include <Adafruit_GFX.h>
#include <Adafruit_SSD1306.h>
#define SCREEN_WIDTH 128 // OLED display width, in pixels
#define SCREEN_HEIGHT 64 // OLED display height, in pixels
// Declaration for an SSD1306 display connected to I2C (SDA, SCL pins)
// The pins for I2C are defined by the Wire-library. 
// On an arduino UNO:       A4(SDA), A5(SCL)
// On an arduino MEGA 2560: 20(SDA), 21(SCL)
// On an arduino LEONARDO:   2(SDA),  3(SCL), ...
#define OLED_RESET     -1 // Reset pin # (or -1 if sharing Arduino reset pin)  NEEDS TO BE -1 FOR D1 MINI
#define SCREEN_ADDRESS 0x3D ///< See datasheet for Address; 0x3D for 128x64, 0x3C for 128x32  DOES NOT SEEM TO MATTER
Adafruit_SSD1306 display(SCREEN_WIDTH, SCREEN_HEIGHT, &Wire, OLED_RESET);


// Analog to Digital converter setup since a D1 Mini only has one Analog in
#include <Wire.h>
#include <Adafruit_ADS1015.h>
// Adafruit_ADS1115 ads;  /* Use this for the 16-bit version */
Adafruit_ADS1015 ads;     /* Use thi for the 12-bit version */


// LED Setup - must come before the painlessMesh so CRGB can be used as a return type
#include <FastLED.h>
// How many leds in your strip?
#define NUM_LEDS 1
// For led chips like Neopixels, which have a data line, ground, and power, you just
// need to define DATA_PIN.  Note that the GPIO and Data pins are laballed separately on ESP8266 boards
#define DATA_PIN D6
// Define the array of leds
CRGB leds[NUM_LEDS];


// Global App Vars
// the timer is used to send commands only once in awhile to not flood the message queue
unsigned long ul_PreviousMillis = 0UL;   // using millis rollver code from http://playground.arduino.cc/Code/TimingRollover
unsigned long ul_Interval = 1000UL;      // send commands at most once a second
//int inputPinR = A0;
//int inputPinG = A1;
//int inputPinB = A2;
int16_t valR = 0;
int16_t valG = 0;
int16_t valB = 0;
int16_t valRold = 0;
int16_t valGold = 0;
int16_t valBold = 0;
// using averaging to debaounce a bit - not perfect
#define NUMSAMPLES 10
int16_t rVals[NUMSAMPLES];
int16_t gVals[NUMSAMPLES];
int16_t bVals[NUMSAMPLES];
int16_t sampleNum = 0;


// Mesh Setup
#include "painlessMesh.h"
#define   MESH_PREFIX     "ColorPicker"
#define   MESH_PASSWORD   "foobarbaz"       // needs to be at leats 8 characters!
#define   MESH_PORT       5555
painlessMesh  mesh;

// called on a new connection - ignore since we will used changedConnection
void newConnectionCallback(uint32_t nodeId) {
    Serial.printf("--> startHere: New Connection, nodeId = %u\n", nodeId);
} // newConnectionCallback

// this gets called when a node is added or removed from the mesh, so set the controller to the node with the lowest chip id
void changedConnectionCallback() {
  Serial.printf("Changed connections %s\n",mesh.subConnectionJson().c_str());
} // changedConnectionCallback


// setup
void setup() {
  Serial.begin(9600);
  
  // SSD1306_SWITCHCAPVCC = generate display voltage from 3.3V internally
  if(!display.begin(SSD1306_SWITCHCAPVCC, 0x3C)) { // Address 0x3C for 128x32
    Serial.println(F("SSD1306 allocation failed"));
    for(;;); // Don't proceed, loop forever
  }
  //display.display();
  //delay(2000); // Pause for 2 seconds - shows Adafruit logo
  display.clearDisplay();
  display.setTextSize(2);             // Draw 2X-scale text
  display.setTextColor(SSD1306_WHITE);  

  FastLED.addLeds<WS2812B, DATA_PIN, RGB>(leds, NUM_LEDS);
  //FastLED.setBrightness(60);
  
  //mesh.setDebugMsgTypes( ERROR | MESH_STATUS | CONNECTION | SYNC | COMMUNICATION | GENERAL | MSG_TYPES | REMOTE ); // all types on
  mesh.setDebugMsgTypes( ERROR | STARTUP );  // set before init() so that you can see startup messages
  mesh.init( MESH_PREFIX, MESH_PASSWORD, MESH_PORT );
  mesh.onNewConnection(&newConnectionCallback);
  mesh.onChangedConnections(&changedConnectionCallback);
  Serial.print("My Node ID: ");
  Serial.println(mesh.getNodeId());

  // start the command send timer
  ul_PreviousMillis = millis();

  // start the ADC converter
  ads.begin();

  // clear out the color val averaging buffers
  for (int i=0; i<NUMSAMPLES; i++) {
    rVals[i] = 0;
    gVals[i] = 0;
    bVals[i] = 0;
  }
} // setup


// loop
void loop() {
  mesh.update();  // run the mesh code

  // read the pots, and save the values - this could be condensed, but perhaps less readable
  valR = map(ads.readADC_SingleEnded(0), 0, 1571, -5, 260);
  valG = map(ads.readADC_SingleEnded(1), 0, 1571, -5, 260);
  valB = map(ads.readADC_SingleEnded(2), 0, 1571, -5, 260);
  valR = constrain(valR, 0, 255);
  valG = constrain(valG, 0, 255);
  valB = constrain(valB, 0, 255);
  rVals[sampleNum] = valR;
  gVals[sampleNum] = valG;
  bVals[sampleNum] = valB;
  sampleNum++;
  if (sampleNum == NUMSAMPLES) sampleNum = 0;

  // get the running average value
  valR = 0;
  valG = 0;
  valB = 0;
  for (int i=0; i<NUMSAMPLES; i++) {
    valR += rVals[i];
    valG += gVals[i];
    valB += bVals[i];
  }
  valR = valR / NUMSAMPLES;
  valG = valG / NUMSAMPLES;
  valB = valB / NUMSAMPLES;
  
  // if there is a change, display it, set the LED and send a message to the receiver
  // the pots fluctuate a lot, so this gets called a lot
  if ((valR != valRold) || (valG != valGold) || (valB != valBold)) {
    display.clearDisplay(); // Clear display buffer
    display.setCursor(0, 0);     // Start at top-left corner
    //display.println("Color");
    display.print("R: ");
    if (valR < 10) display.print(" ");
    if (valR < 100) display.print(" ");
    display.println(valR);
    display.print("G: ");
    if (valG < 10) display.print(" ");
    if (valG < 100) display.print(" ");
    display.println(valG);
    display.print("B: ");
    if (valB < 10) display.print(" ");
    if (valB < 100) display.print(" ");
    display.print(valB);
    display.display();

    // update the LED
    for(uint16_t i=0; i<NUM_LEDS; i++) leds[i] = CRGB(valR, valG, valB);
    FastLED.show();

    // send a message to the receiver no more than once a second
    unsigned long ul_CurrentMillis = millis();  
    if (ul_CurrentMillis - ul_PreviousMillis > ul_Interval) {
      sendMessage(valR, valG, valB); 
      ul_PreviousMillis = millis();
    }
  }

  // save the current values
  valRold = valR;
  valGold = valG;
  valBold = valB;
} // loop


// send a broadcast message to all the nodes specifying the new animation mode for all of them
void sendMessage(int16_t r, int16_t g, int16_t b) {
  String msg;

  // send the message only if there is a receiver - seems to speed up synchronization
  if (mesh.getNodeList().size() > 0) {
    //Serial.printf("Num nodes: %d\n", nodes.size());
    Serial.print("Sending message to change display_mode to ");
    msg = String((r*256*256) + (g * 256) + b);
    Serial.println(msg);
    mesh.sendBroadcast( msg );
  } else {
    // blink if we do not have a connection yet
    for(uint16_t i=0; i<NUM_LEDS; i++) leds[i] = CRGB(0, 0, 0);
    FastLED.show();
    Serial.println("waiting for receiver");
  }
} // sendMessage
